@@ -3,7 +3,7 @@ use crate::utils::return_invalid_data_if_not_equal;
 use crate::{LpcmKind, Metadata};
 use std::io::Result;
 
-#[derive(Clone, Copy, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct WavMetadata {
     /// Number of sample frames
     pub frames: u64,
@@ -13,6 +13,14 @@ pub struct WavMetadata {
     pub channels: u16,
     /// Samples per sec
     pub samples_per_sec: f64,
+    /// List data
+    pub list: Vec<Chunk>,
+}
+
+#[derive(Clone, Debug, PartialEq)]
+pub struct Chunk {
+    pub fourcc: String,
+    pub data: Vec<u8>,
 }
 
 impl Metadata for WavMetadata {}
@@ -23,12 +31,14 @@ impl WavMetadata {
         lpcm_kind: LpcmKind,
         channels: u16,
         samples_per_sec: f64,
+        list: Vec<Chunk>,
     ) -> Self {
         Self {
             frames,
             lpcm_kind,
             channels,
             samples_per_sec,
+            list,
         }
     }
 
@@ -118,55 +128,87 @@ impl WavMetadata {
             Ok(())
         };
 
-        // Fmt chunk
+        // Declare
+        let mut format_tag: u16 = 0;
+        let mut channels: u16 = 0;
+        let mut samples_per_sec: u32 = 0;
+        let mut avg_bytes_per_sec: u32 = 0;
+        let mut block_align: u16 = 0;
+        let mut bits_per_sample: u16 = 0;
+        let mut list: Vec<Chunk> = vec![];
+
+        // Read chunks
         loop {
-            match check_fourcc(reader, "fmt ") {
-                Ok(_) => {
-                    let fmt_size: u32 = reader.read_le()?;
-                    return_invalid_data_if_not_equal(fmt_size, 16)?;
+            let fourcc = reader.read_string::<4>()?;
+            match fourcc.as_ref() {
+                "LIST" => {
+                    let list_size: u32 = reader.read_le()?;
+                    check_fourcc(reader, "INFO")?;
+                    let mut curr_size: u32 = 4;
+                    while list_size != curr_size {
+                        let fourcc = reader.read_string::<4>()?;
+                        let child_size: u32 = reader.read_le()?;
+                        let mut data = vec![0; child_size as usize];
+                        reader.read_exact(&mut data)?;
+                        list.push(Chunk { fourcc, data });
+                        curr_size += 8 + child_size;
 
-                    let format_tag: u16 = reader.read_le()?;
-                    let channels: u16 = reader.read_le()?;
-                    let samples_per_sec: u32 = reader.read_le()?;
-                    let avg_bytes_per_sec: u32 = reader.read_le()?;
-                    let block_align: u16 = reader.read_le()?;
-                    let bits_per_sample: u16 = reader.read_le()?;
-
-                    // Data chunk
-                    loop {
-                        match check_fourcc(reader, "data") {
-                            Ok(_) => {
-                                let data_size: u32 = reader.read_le()?;
-
-                                let frames =
-                                    Self::calculate_frames(data_size, channels, bits_per_sample);
-
-                                let wav_metadata = Self {
-                                    frames,
-                                    lpcm_kind: LpcmKind::from_format_tag_and_bits_per_sample(
-                                        format_tag,
-                                        bits_per_sample,
-                                    ),
-                                    channels,
-                                    samples_per_sec: samples_per_sec as f64,
-                                };
-
-                                return_invalid_data_if_not_equal(
-                                    avg_bytes_per_sec,
-                                    wav_metadata.avg_bytes_per_sec(),
-                                )?;
-                                return_invalid_data_if_not_equal(
-                                    block_align,
-                                    wav_metadata.block_align(),
-                                )?;
-
-                                return Ok(wav_metadata);
-                            }
-                            Err(_) => other_chunk(reader)?,
+                        // Chunk must be an even number of bytes
+                        if child_size % 2 == 1 {
+                            let mut buf = [0];
+                            reader.read_exact(&mut buf)?;
+                            curr_size += 1;
                         }
                     }
                 }
-                Err(_) => other_chunk(reader)?,
+                "fmt " => {
+                    let fmt_size: u32 = reader.read_le()?;
+
+                    format_tag = reader.read_le()?;
+                    channels = reader.read_le()?;
+                    samples_per_sec = reader.read_le()?;
+                    avg_bytes_per_sec = reader.read_le()?;
+                    block_align = reader.read_le()?;
+                    bits_per_sample = reader.read_le()?;
+
+                    match fmt_size {
+                        16 => {}
+                        18 => {
+                            let _cb_size: u16 = reader.read_le()?;
+                        }
+                        19.. => {
+                            let cb_size: u16 = reader.read_le()?;
+                            let mut skip = vec![0; cb_size as usize];
+                            reader.read_exact(&mut skip)?;
+                        }
+                        _ => unimplemented!("fmt size"),
+                    }
+                }
+                "data" => {
+                    let data_size: u32 = reader.read_le()?;
+
+                    let frames = Self::calculate_frames(data_size, channels, bits_per_sample);
+
+                    let wav_metadata = Self {
+                        frames,
+                        lpcm_kind: LpcmKind::from_format_tag_and_bits_per_sample(
+                            format_tag,
+                            bits_per_sample,
+                        ),
+                        channels,
+                        samples_per_sec: samples_per_sec as f64,
+                        list,
+                    };
+
+                    return_invalid_data_if_not_equal(
+                        avg_bytes_per_sec,
+                        wav_metadata.avg_bytes_per_sec(),
+                    )?;
+                    return_invalid_data_if_not_equal(block_align, wav_metadata.block_align())?;
+
+                    return Ok(wav_metadata);
+                }
+                _ => other_chunk(reader)?,
             }
         }
     }
@@ -205,6 +247,7 @@ mod tests {
             lpcm_kind: LpcmKind::F32LE,
             channels: 1,
             samples_per_sec: 48000.0,
+            list: vec![],
         };
         assert_eq!(metadata.secs(), 1);
         assert_eq!(metadata.millis(), 1_000);
@@ -215,6 +258,7 @@ mod tests {
             lpcm_kind: LpcmKind::F32LE,
             channels: 1,
             samples_per_sec: 48000.0,
+            list: vec![],
         };
         assert_eq!(metadata.secs(), 1);
         assert_eq!(metadata.millis(), 1_000);
@@ -225,6 +269,7 @@ mod tests {
             lpcm_kind: LpcmKind::F32LE,
             channels: 1,
             samples_per_sec: 48000.0,
+            list: vec![],
         };
         assert_eq!(metadata.secs(), 1);
         assert_eq!(metadata.millis(), 1_999);
@@ -243,6 +288,7 @@ mod tests {
             lpcm_kind,
             channels: 1,
             samples_per_sec,
+            list: vec![],
         };
         assert_eq!(metadata.format_tag(), 3);
         assert_eq!(metadata.channels(), 1);
@@ -258,6 +304,7 @@ mod tests {
             lpcm_kind,
             channels: 1,
             samples_per_sec,
+            list: vec![],
         };
         assert_eq!(metadata.format_tag(), 3);
         assert_eq!(metadata.channels(), 1);
@@ -275,6 +322,7 @@ mod tests {
             lpcm_kind,
             channels: 2,
             samples_per_sec,
+            list: vec![],
         };
         assert_eq!(metadata.format_tag(), 3);
         assert_eq!(metadata.channels(), 2);
@@ -301,6 +349,7 @@ mod tests {
             lpcm_kind,
             channels: 1,
             samples_per_sec: 44100.0,
+            list: vec![],
         };
         assert_eq!(val, expect);
 
@@ -368,6 +417,7 @@ mod tests {
             lpcm_kind,
             channels: 2,
             samples_per_sec: 44100.0,
+            list: vec![],
         };
         assert_eq!(val, expect);
 
@@ -384,6 +434,47 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn sakana() -> Result<()> {
+        let mut data: &[u8] = &[
+            0x52, 0x49, 0x46, 0x46, 0xE6, 0xAF, 0x87, 0x02, 0x57, 0x41, 0x56, 0x45, 0x4C, 0x49,
+            0x53, 0x54, 0xE8, 0x00, 0x00, 0x00, 0x49, 0x4E, 0x46, 0x4F, 0x49, 0x41, 0x52, 0x54,
+            0x0F, 0x00, 0x00, 0x00, 0x83, 0x54, 0x83, 0x4A, 0x83, 0x69, 0x83, 0x4E, 0x83, 0x56,
+            0x83, 0x87, 0x83, 0x93, 0x00, 0x00, 0x49, 0x4E, 0x41, 0x4D, 0x27, 0x00, 0x00, 0x00,
+            0x81, 0x77, 0x83, 0x6F, 0x83, 0x62, 0x83, 0x6E, 0x82, 0xCC, 0x90, 0xF9, 0x97, 0xA5,
+            0x82, 0xF0, 0x96, 0xE9, 0x82, 0xC9, 0x92, 0xAE, 0x82, 0xA2, 0x82, 0xBD, 0x82, 0xB9,
+            0x82, 0xA2, 0x82, 0xC5, 0x82, 0xB7, 0x81, 0x42, 0x81, 0x78, 0x00, 0x00, 0x49, 0x50,
+            0x52, 0x44, 0x15, 0x00, 0x00, 0x00, 0x44, 0x6F, 0x63, 0x75, 0x6D, 0x65, 0x6E, 0x74,
+            0x61, 0x4C, 0x79, 0x20, 0x5B, 0x8F, 0x89, 0x89, 0xF1, 0x94, 0xD5, 0x5D, 0x00, 0x00,
+            0x49, 0x47, 0x4E, 0x52, 0x14, 0x00, 0x00, 0x00, 0x49, 0x6E, 0x64, 0x69, 0x65, 0x20,
+            0x2F, 0x20, 0x41, 0x6C, 0x74, 0x65, 0x72, 0x6E, 0x61, 0x74, 0x69, 0x76, 0x65, 0x00,
+            0x49, 0x54, 0x4F, 0x43, 0x4E, 0x00, 0x00, 0x00, 0x44, 0x2B, 0x39, 0x36, 0x2B, 0x44,
+            0x30, 0x32, 0x2B, 0x35, 0x36, 0x36, 0x46, 0x2B, 0x39, 0x43, 0x34, 0x43, 0x2B, 0x46,
+            0x41, 0x39, 0x31, 0x2B, 0x31, 0x34, 0x31, 0x38, 0x36, 0x2B, 0x31, 0x38, 0x30, 0x41,
+            0x31, 0x2B, 0x31, 0x45, 0x42, 0x43, 0x37, 0x2B, 0x32, 0x32, 0x45, 0x37, 0x33, 0x2B,
+            0x32, 0x37, 0x37, 0x32, 0x30, 0x2B, 0x32, 0x42, 0x44, 0x39, 0x46, 0x2B, 0x33, 0x30,
+            0x41, 0x45, 0x44, 0x2B, 0x33, 0x35, 0x46, 0x41, 0x37, 0x2B, 0x33, 0x42, 0x42, 0x41,
+            0x35, 0x00, 0x49, 0x54, 0x52, 0x4B, 0x03, 0x00, 0x00, 0x00, 0x31, 0x30, 0x00, 0x00,
+            0x66, 0x6D, 0x74, 0x20, 0x12, 0x00, 0x00, 0x00, 0x01, 0x00, 0x02, 0x00, 0x44, 0xAC,
+            0x00, 0x00, 0x10, 0xB1, 0x02, 0x00, 0x04, 0x00, 0x10, 0x00, 0x00, 0x00, 0x64, 0x61,
+            0x74, 0x61, 0xD0, 0xAE, 0x87, 0x02,
+        ];
+
+        let metadata = WavMetadata::read(&mut data)?;
+        let list = metadata.list;
+
+        let iart = list.into_iter().find(|e| e.fourcc == "IART").unwrap().data;
+        assert_eq!(
+            &iart,
+            &[
+                0x83, 0x54, 0x83, 0x4A, 0x83, 0x69, 0x83, 0x4E, 0x83, 0x56, 0x83, 0x87, 0x83, 0x93,
+                0x00
+            ]
+        );
+
+        Ok(())
+    }
+
     #[test]
     fn write_and_read() -> Result<()> {
         let mut v = Vec::new();
@@ -392,6 +483,7 @@ mod tests {
             lpcm_kind: LpcmKind::F32LE,
             channels: 1,
             samples_per_sec: 44100.0,
+            list: vec![],
         };
         let mut data: &[u8] = &[
             0x52, 0x49, 0x46, 0x46, 0x44, 0x62, 0x05, 0x00, 0x57, 0x41, 0x56, 0x45, 0x66, 0x6D,
